@@ -1,36 +1,39 @@
 import numpy as np
 from pathlib import Path
+from predictit.misc import traceback_warning
 
 
-def train(sequentions, layers='default', predicts=7, epochs=200, load_trained_model=0, update_trained_model=1, save_model=1,
-          saved_model_path_string='stored_models', optimizer='adam', loss='mse', verbose=0, used_metrics='accuracy', timedistributed=0, batch_size=64):
-    """Tensorflow model. Neural nets - LSTM or MLP (just dense layers). Layers are customizable with arguments.
+def train(sequentions, layers='default', predicts=7, epochs=200, already_trained=0, train_next=1, save=1, saved_model_path_string='stored_models', optimizer='adam', loss='mse', verbose=0, metrics='accuracy', timedistributed=0, batch_size=64):
+    """Tensorflow LSTM model.
 
     Args:
         sequentions (tuple(np.ndarray, np.ndarray, np.ndarray)) - Tuple (X, y, x_input) of input train vectors X, train outputs y, and input for prediction x_input
-        layers (tuple) - Tuple of tuples of layer name (e.g. 'lstm') and layer params dict (e.g. {'units': 7, 'activation': 'relu'}). Check default layres tuple here or config example.
+        model (str) - Type of neural network. 'lstm', 'bidirectional_lstm' or 'mlp'. Defaults to 'lstm'.
         predicts (int, optional): Number of predicted values. Defaults to 7.
         epochs (int, optional): Number of epochs to evaluate. Defaults to 100.
-        load_trained_model: If 1, load model from disk. Most of time is spend on training, so if loaded and not updated, it's very fast.
-        update_trained_model: If load_trained_model, it's updated with new input.
-        save_model: If 1, save model on disk on saved_model_path_string.
-        saved_model_path_string: Full path to saved model with name. E.g. '/home/dan/mymodel.h5. If 'stored_models', then it's save to library folder models/stored_models.
+        units (int, optional): Number of neural units. Defaults to 50.
+        layers (int): Number ob neuron layers. Defaults to 3.
+        dropout (int, optional): Whether use dropout layer. Use values around 0.1. If 0, no dropout. Defaults to 0.
+        timedistributed (int, optional): Whether use timedistributed layer. Defaults to 0.
         optimizer (str, optional): Used optimizer. Defaults to 'adam'.
         loss (str, optional): Loss function. Defaults to 'mse'.
         verbose (int, optional): Wheter display details. Defaults to 0.
-        used_metrics (str, optional): Used metrics. 'accuracy' or 'mape' Defaults to 'accuracy'.
+        activation (str, optional): Used activation function. Defaults to 'relu'.
+        metrics (str, optional): Used metrics. Defaults to 'accuracy'.
 
+    Layers kwargs:
+        BatchNormalization: momentum=0.99, epsilon=0.001
     Returns:
-        model: Trained model object.
+        np.ndarray: Predictions of input time series.
     """
 
     import keras
     import tensorflow as tf
 
     if layers == 'default':
-        layers = (('lstm', {'units': 32, 'activation': 'relu', 'return_sequences': 1}),
-                  ('dropout', {'rate': 0.1}),
-                  ('lstm', {'units': 7, 'activation': 'relu'}))
+        layers = [['lstm', {'units': 32, 'activation': 'relu', 'return_sequences': 1}],
+                  ['dropout', {'rate': 0.1}],
+                  ['lstm', {'units': 7, 'activation': 'relu'}]]
 
     X = sequentions[0]
     y = sequentions[1]
@@ -38,34 +41,22 @@ def train(sequentions, layers='default', predicts=7, epochs=200, load_trained_mo
 
     models = {'dense': keras.layers.Dense, 'lstm': keras.layers.LSTM, 'mlp': keras.layers.Dense, 'gru': keras.layers.GRU, 'conv2d': keras.layers.Conv2D, 'rnn': keras.layers.SimpleRNN, 'convlstm2d': keras.layers.ConvLSTM2D, 'dropout': keras.layers.Dropout, 'batchnormalization': keras.layers.BatchNormalization}
 
-    if used_metrics == 'accuracy':
+    if metrics == 'accuracy':
         metrics = [tf.keras.metrics.Accuracy()]
-    elif used_metrics == 'mape':
-        metrics = [tf.keras.metrics.MeanAbsolutePercentageError()]
+
+    if layers[0][0] == 'lstm':
+        if X.ndim == 2:
+            X = X.reshape(X.shape[0], X.shape[1], 1)
+        layers[0][1]['input_shape'] = (X.shape[1], X.shape[2])
+
+    if layers[0][0] == 'dense':
+        layers[0][1]['input_shape'] = (X.shape[1],)
+        assert (X.ndim != 3), 'For dense first layer only univariate data supported (e.g. shape = (n_samples, n_features))- if ndim > 2: serialize first.'
 
     if saved_model_path_string == 'stored_models':
-        saved_model_path_string = str(Path(__file__).resolve().parent / 'stored_models' / 'tensorflow.h5')
+        saved_model_path_string = str(Path(__file__).resolve().parents[0] / 'stored_models' / 'tensorflow.h5')
 
-    if load_trained_model:
-        try:
-            model = tf.keras.models.load_model(saved_model_path_string)
-            model.load_weights(saved_model_path_string)
-
-        except Exception:
-            raise NameError("Model is not saved, first saveit = 1 in config")
-
-        if update_trained_model:
-            model.fit(X, y, epochs=epochs, batch_size=64, verbose=verbose)
-
-    else:
-        if layers[0][0] == 'lstm':
-            if X.ndim == 2:
-                X = X.reshape(X.shape[0], X.shape[1], 1)
-            layers[0][1]['input_shape'] = (X.shape[1], X.shape[2])
-
-        if layers[0][0] == 'dense':
-            layers[0][1]['input_shape'] = (X.shape[1],)
-            assert (X.ndim != 3), 'For dense first layer only univariate data supported (e.g. shape = (n_samples, n_features))- if ndim > 2: serialize first.'
+    if not already_trained:
 
         model = keras.models.Sequential()
 
@@ -81,8 +72,18 @@ def train(sequentions, layers='default', predicts=7, epochs=200, load_trained_mo
         model.compile(optimizer=optimizer, loss=loss, metrics=metrics)
         model.fit(X, y, epochs=epochs, batch_size=64, verbose=verbose)
 
-    if save_model == 1:
-        model.save(saved_model_path_string)
+        if save == 1:
+            model.save(saved_model_path_string)
+
+    if already_trained == 1:
+        try:
+            model = tf.keras.models.load_model(saved_model_path_string)
+            model.load_weights(saved_model_path_string)
+            if train_next:
+                model.fit(X, y, epochs=epochs, batch_size=64, verbose=verbose)
+
+        except Exception:
+            raise NameError("Model is not saved, first saveit = 1 in config")
 
     model.layers_0_0 = layers[0][0]
 
@@ -93,16 +94,6 @@ def train(sequentions, layers='default', predicts=7, epochs=200, load_trained_mo
 
 
 def predict(x_input, model, predicts=7):
-    """Function that creates predictions from trained model and input data.
-
-    Args:
-        x_input (np.ndarray): Time series data
-        model (model): Trained model object from train function.
-        predicts (int, optional): Number of predicted values. Defaults to 7.
-
-    Returns:
-        np.ndarray: Array of predicted results
-    """
 
     if model.X_ndim == 2 and model.layers_0_0 == 'lstm':
         x_input = x_input.reshape(1, x_input.shape[1], x_input.shape[0])
@@ -125,11 +116,7 @@ def predict(x_input, model, predicts=7):
 
 
 def get_optimizers_loses_activations():
-    """Return list of tensorflow optimizers. It's used by optimize function.
 
-    Returns:
-        list: List of tensorflow optimizers.
-    """
     from keras import optimizers
 
     sgd = optimizers.SGD(lr=0.01, decay=1e-6, momentum=0.9, nesterov=True)
@@ -140,4 +127,6 @@ def get_optimizers_loses_activations():
     adamax = optimizers.Adamax(lr=0.002, beta_1=0.9, beta_2=0.999, epsilon=None, decay=0.0)
     nadam = optimizers.Nadam(lr=0.002, beta_1=0.9, beta_2=0.999, epsilon=None, schedule_decay=0.004)
 
-    return [sgd, rmsprop, adagrad, adadelta, adam, adamax, nadam]
+    lstm_optimizers = [sgd, rmsprop, adagrad, adadelta, adam, adamax, nadam]
+
+    return lstm_optimizers
